#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

import os
import re


MANUAL_INDEX_PATH = "/usr/local/share/manual/index.html"
PKG_MANUAL_INDEX_PATH = "/usr/local/share/manual/package/index.html"
MANUAL_DATA_PATH = os.path.dirname(os.path.abspath(__file__))
MANUAL_LIST_PATH = os.path.join(MANUAL_DATA_PATH, "manual_list.html")
MANUAL_BODY_PATH = os.path.join(MANUAL_DATA_PATH, "manual_body.html")
MANUAL_LOG_PATH_LIST = []

CONTAINER_ID_RE = 'id\s*=\s*"container"'
CONTENT_ID_RE = 'id\s*=\s*"content"'
INDEX_RE = "<!--\s*index\s*=\s*([0-9]+)\s*[^>]*>"

LOG_SECTION_LINE_RE = """.*<section id="log">.*"""
LOG_SECTION_END_RE = """.*</section>.*"""
CATEGORY_LINE_RE = ".*<h2>.*</h2>.*"
LOG_CODE_RE = "<h3>([0-9]+)-[0-9]+</h3>"


def _get_file_data(file_path):
    """Return data in "file_path"."""

    try:
        with open(file_path) as file_handle:
            return file_handle.read()
    except:
        pass
    return ""


def _get_temp_path(man_path):
    """Return temporary path for "man_path"."""

    return man_path + "." + str(os.getpid())


def _save_file_data(file_path, data):
    """Save "data" to "file_path"."""

    temp_path = _get_temp_path(file_path)
    with open(temp_path, "w") as temp_file:
        temp_file.write(data)

    os.rename(temp_path, file_path)


def _get_index_no(data):
    """Return temporary path for "man_path"."""

    match = re.search(INDEX_RE, data)
    if not match:
        return 0

    return int(match.group(1))


def _insert_pkg_manual(man_data, pkg_man_data, start_re):
    """Insert "pkg_man_data" in "man_data"."""

    index_no = _get_index_no(pkg_man_data)

    start_match = re.search(start_re, man_data)
    if not start_match:
        return man_data

    idx = start_match.end(0)
    while True:
        match = re.search(INDEX_RE, man_data[idx:])
        if not match:
            return man_data

        try:
            if index_no < int(match.group(1)):
                insert_idx = idx + match.start(0)
                man_data = man_data[:insert_idx] + pkg_man_data + \
                                man_data[insert_idx:]
                return man_data

            idx += match.end(0)
        except:
            # Ignore error
            continue


def _remove_pkg_manual(man_data, pkg_man_data, start_re):
    """remove "pkg_man_data" from "man_data"."""

    index_no = _get_index_no(pkg_man_data)

    start_match = re.search(start_re, man_data)
    if not start_match:
        return man_data

    remove_start = 0
    idx = start_match.end(0)
    while True:
        match = re.search(INDEX_RE, man_data[idx:])
        if not match:
            return man_data

        try:
            if index_no == int(match.group(1)):
                remove_start = idx + match.start(0)
            elif remove_start and index_no < int(match.group(1)):
                remove_end = idx + match.start(0)
                man_data = man_data[:remove_start] + man_data[remove_end:]
                return man_data

            idx += match.end(0)
        except:
            # Ignore error
            continue


def _to_category_code(code_high):
    """Return category code corresponding "code_high"."""

    return int(code_high) / 100 * 100


def _get_next_category_code(data, start=0, end=-1):
    """Return category code which is found at first in "data[start:end]"."""

    match = re.search(LOG_CODE_RE, data[start:end])
    if not match:
        return None

    return _to_category_code(match.group(1))


def _get_log_section_range(man_data):
    """Return the start and end position in "man_data"."""

    start_match = re.search(LOG_SECTION_LINE_RE, man_data)
    if not start_match:
        return (None, None)

    idx = start_match.end(0)
    end_match = re.search(LOG_SECTION_END_RE, man_data[idx:])
    if not end_match:
        return (None, None)

    return (idx, idx + end_match.start(0))


def _get_next_category_line_range(data, start, end):
    """Return the start and end position of next category line
       in "data[start:end]"."""

    match = re.search(CATEGORY_LINE_RE, data[start:end])
    if match:
        return (start + match.start(0), start + match.end(0))

    return (end, end)


def _get_next_category_line_end_position(data, start, end):
    """Return next category line end position in "data[start:end]"."""

    match = re.search(CATEGORY_LINE_RE, data[start:end])
    if match:
        return start + match.end(0)

    return end


def _insert_pkg_log(man_data, pkg_log_data):
    """Insert each log entry in "pkg_log_data" in "man_data"."""

    pkg_category_code = _get_next_category_code(pkg_log_data)
    if pkg_category_code is None:
        return man_data

    (idx, end) = _get_log_section_range(man_data)
    if not idx or not end:
        return man_data

    while idx < end:
        (insert_idx, next_idx) = \
                _get_next_category_line_range(man_data, idx, end)
        category_code = _get_next_category_code(man_data, next_idx, end)

        if category_code is not None:
            if category_code < pkg_category_code:
                idx = next_idx
                continue

        man_data = man_data[:insert_idx] + pkg_log_data + man_data[insert_idx:]
        return man_data

    return man_data


def _remove_pkg_log(man_data, pkg_log_data):
    """remove each log entry in "pkg_log_data" from "man_data"."""

    pkg_category_code = _get_next_category_code(pkg_log_data)
    if pkg_category_code is None:
        return man_data

    (idx, end) = _get_log_section_range(man_data)
    if not idx or not end:
        return man_data

    while idx < end:
        (remove_idx, next_idx) = \
                _get_next_category_line_range(man_data, idx, end)
        category_code = _get_next_category_code(man_data, next_idx, end)

        if category_code != pkg_category_code:
            idx = next_idx
            continue

        (remove_end, unused) = \
                _get_next_category_line_range(man_data, next_idx, end)
        man_data = man_data[:remove_idx] + man_data[remove_end:]
        return man_data

    return man_data


def install():
    """Install manual."""

    pkg_man_data = _get_file_data(PKG_MANUAL_INDEX_PATH)
    list_data = _get_file_data(MANUAL_LIST_PATH)
    body_data = _get_file_data(MANUAL_BODY_PATH)

    if pkg_man_data:
        if list_data:
            pkg_man_data = _insert_pkg_manual(pkg_man_data, list_data,
                                                CONTAINER_ID_RE)

        if body_data:
            pkg_man_data = _insert_pkg_manual(pkg_man_data, body_data,
                                                CONTENT_ID_RE)

        _save_file_data(PKG_MANUAL_INDEX_PATH, pkg_man_data)


    man_data = _get_file_data(MANUAL_INDEX_PATH)
    if not man_data:
        return

    for manual_log_path in MANUAL_LOG_PATH_LIST:
        pkg_log_data = _get_file_data(manual_log_path)
        if pkg_log_data:
            man_data = _insert_pkg_log(man_data, pkg_log_data)

    _save_file_data(MANUAL_INDEX_PATH, man_data)


def uninstall():
    """uninstall manual."""

    pkg_man_data = _get_file_data(PKG_MANUAL_INDEX_PATH)
    list_data = _get_file_data(MANUAL_LIST_PATH)
    body_data = _get_file_data(MANUAL_BODY_PATH)

    if pkg_man_data:
        if list_data:
            pkg_man_data = _remove_pkg_manual(pkg_man_data, list_data,
                                                CONTAINER_ID_RE)

        if body_data:
            pkg_man_data = _remove_pkg_manual(pkg_man_data, body_data,
                                                CONTENT_ID_RE)

        _save_file_data(PKG_MANUAL_INDEX_PATH, pkg_man_data)

    man_data = _get_file_data(MANUAL_INDEX_PATH)
    if not man_data:
        return

    for manual_log_path in MANUAL_LOG_PATH_LIST:
        pkg_log_data = _get_file_data(manual_log_path)
        if pkg_log_data:
            man_data = _remove_pkg_log(man_data, pkg_log_data)

    _save_file_data(MANUAL_INDEX_PATH, man_data)



def update():
    """Update manual."""

    uninstall()
    install()
