#!/usr/bin/env python
# -*- coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import absolute_import
from __future__ import division

from ionas.request import Request, Entry, EntryList, XmlConfNode
from product_conf import SUDO_CMD
from fal_info import Results

from fal_info import FAL_SYNC_PATH

from . import _utils
from .. import storages


__empty = object()


def _dict_to_node(_dict, _tagname):
    from ionas.request import XmlConfNode as Node
    spk = _dict['spk']
    for storage in storages.storages:
        if storage['pk'] == spk:
            _node = storage['dict_to_node'](_dict, _tagname)
            _node.append_child('type', text=spk)
            return _node
    return Node(_tagname)


def _node_to_dict(node):
    spk = node.get_node('type').get_text()
    for storage in storages.storages:
        if storage['pk'] == spk:
            _dict = storage['node_to_dict'](node)
            _dict['spk'] = spk
            return _dict
    return {}


def _call_load():
    entry = Entry('load', """<sync />""")
    entry = _utils.send_entry(FAL_SYNC_PATH, entry)

    result = entry.get_result()

    syncconfs = []
    try:
        for syncconf_node in entry.get().get_node_list('syncconflist/syncconf'):
            try:
                syncconfs.append({
                    'id': syncconf_node.get_node('id').get_text(),
                    'name': syncconf_node.get_node('name').get_text(),
                    'src': _node_to_dict(syncconf_node.get_node('source')),
                    'dst': _node_to_dict(syncconf_node.get_node('destination')),
                    'ignores': [{
                        'type': ignore_node.get_node('type').get_text(),
                        'pattern': ignore_node.get_node('pattern').get_text(),
                    } for ignore_node in syncconf_node.get_node_list('ignorelist/ignore')],
                    'direction': syncconf_node.get_node('direction').get_text(),
                    'interval': syncconf_node.get_node('interval').get_text(),
                    'maxfilesize': syncconf_node.get_node('maxfilesize').get_text(),
                })
            except:
                continue
    except:
        pass

    return {
        'is_succeeded': result in (Results.OK,),
        'data': syncconfs,
    }


def _call_add_or_modify(operation, parameters, messages):
    entry = Entry(operation, """<sync><syncconf /></sync>""")
    sync_node = entry.get()
    syncconf_node = sync_node.get_node('syncconf')
    if operation == 'modify':
        syncconf_node.append_child('id', text=parameters['id'])
    syncconf_node.append_child('name', text=parameters['name'])
    syncconf_node.append_child(_dict_to_node(parameters['src'], 'source'))
    syncconf_node.append_child(_dict_to_node(parameters['dst'], 'destination'))
    syncconf_node.append_child('direction', text=parameters['direction'])
    syncconf_node.append_child('interval', text=str(parameters['interval']))
    syncconf_node.append_child(
        'maxfilesize',
        text=str(parameters['maxfilesize']) if parameters['maxfilesize'] else '',
    )
    syncconf_node.append_child('ignorelist')
    ignorelist_node = syncconf_node.get_node('ignorelist')
    for ignore in parameters['ignores']:
        ignore_node = XmlConfNode('ignore')
        ignore_node.append_child('type', text=ignore['type'])
        ignore_node.append_child('pattern', text=ignore['pattern'])
        ignorelist_node.append_child(ignore_node)

    entry = _utils.send_entry(FAL_SYNC_PATH, entry)

    result = entry.get_result()

    constname, arglist = messages.get(result, messages['__else__'])
    message = _utils.gettext(constname, arglist=arglist)
    _utils.logging(constname, arglist)

    return {
        'is_succeeded': result in (Results.OK,),
        'message': message,
    }


def _call_add(parameters):
    return _call_add_or_modify('add', parameters, {
        Results.OK: ['MSG_SYNC_ADD', {'name': parameters['name']}],
        '__else__': ['ERR_SYNC_ADD', {'name': parameters['name']}],
    })


def _call_modify(parameters):
    return _call_add_or_modify('modify', parameters, {
        Results.OK: ['MSG_SYNC_ADD', {'name': parameters['name']}],
        '__else__': ['ERR_SYNC_ADD', {'name': parameters['name']}],
    })


def _call_delete(syncconf_id):
    name = _obtain_name(syncconf_id, default='')

    entry = Entry('delete', """<sync><syncconf /></sync>""")
    sync_node = entry.get()
    syncconf_node = sync_node.get_node('syncconf')
    syncconf_node.append_child('id', text=syncconf_id)

    entry = _utils.send_entry(FAL_SYNC_PATH, entry)

    result = entry.get_result()

    messages = {
        Results.OK: ['MSG_SYNC_DEL', {"name": name}],
        '__else__': ['ERR_SYNC_DEL', {"name": name}],
    }
    constname, arglist = messages.get(result, messages['__else__'])
    message = _utils.gettext(constname, arglist=arglist)
    _utils.logging(constname, arglist)

    return {
        'is_succeeded': result in (Results.OK,),
        'message': message,
    }


def _obtain_name(syncconf_id, default=__empty):
    for syncconf in obtain_syncconfs():
        if syncconf['id'] == syncconf_id:
            return syncconf['name']
    if default is __empty:
        raise Exception()  # TODO
    return default


def obtain_syncconfs():
    return _call_load()['data']


def create_syncconf(parameters):
    return _call_add(parameters)


def update_syncconf(parameters):
    return _call_modify(parameters)


def delete_syncconf(syncconf_id):
    return _call_delete(syncconf_id)
